package it.amattioli.locate;

import it.amattioli.common.properties.Derived;
import it.amattioli.common.properties.PropertyChangeSupport;
import it.amattioli.dominate.hibernate.validators.NotBlank;
import it.amattioli.dominate.util.PropertyChangeEmitter;

import java.beans.PropertyChangeListener;
import java.util.Collection;
import java.util.Collections;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.apache.commons.lang.builder.HashCodeBuilder;
import org.hibernate.validator.NotNull;

public class Address implements PropertyChangeEmitter {
	private PropertyChangeSupport pChange = new PropertyChangeSupport(this);
	private String street;
	private Place place;
	private String zipCode;

	protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {
		pChange.firePropertyChange(propertyName, oldValue, newValue);
	}

	public void addPropertyChangeListener(PropertyChangeListener listener) {
	    pChange.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
	    pChange.removePropertyChangeListener(listener);
	}
	
	@NotBlank
	public String getStreet() {
		return street;
	}

	public void setStreet(String street) {
		String old = getStreet();
		this.street = street;
		firePropertyChange("street", old, street);
	}

	@NotNull
	public Place getPlace() {
		return place;
	}

	public void setPlace(Place place) {
		Place old = getPlace();
		this.place = place;
		firePropertyChange("place", old, place);
	}
	
	@Derived(dependsOn="place")
	public Place getProvince() {
		Place place = getPlace();
		if (place == null) {
			return null;
		}
		Place province =  place.getParent(PlaceType.PROVINCE);
		if (province == null) {
			return null;
		}
		return province;
	}

	public String getZipCode() {
		return zipCode;
	}

	public void setZipCode(String zipCode) {
		String old = getZipCode();
		this.zipCode = zipCode;
		firePropertyChange("zipCode", old, zipCode);
	}
	
	@Derived(dependsOn="place")
	public Collection<String> getZipCodeValues() {
		if (getPlace() == null) {
			return Collections.emptySet();
		}
		return getPlace().getZipCodes();
	}

	@Override
	public Address clone() throws CloneNotSupportedException {
		return (Address)super.clone();
	}

	@Override
	public boolean equals(Object obj) {
		return EqualsBuilder.reflectionEquals(this, obj);
	}

	@Override
	public int hashCode() {
		return HashCodeBuilder.reflectionHashCode(this);
	}

}
