package it.amattioli.locate;

import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.apache.commons.beanutils.PropertyUtils;

import it.amattioli.dominate.Entity;
import it.amattioli.dominate.specifications.AbstractSpecification;
import it.amattioli.dominate.specifications.Assembler;
import it.amattioli.dominate.specifications.hibernate.HqlAssembler;

public class CollectionSpecification<T extends Entity<?>> extends AbstractSpecification<T> {
	private Object value;
	private String propertyName;

	public CollectionSpecification(String propertyName) {
		super();
		this.propertyName = propertyName;
	}

	@Override
	public void assembleQuery(Assembler assembler) {
		if (wasSet()) {	
			HqlAssembler hqlAssembler = (HqlAssembler)assembler;
			hqlAssembler.newCriteria();
			hqlAssembler.append(":param0 in elements");
			hqlAssembler.append("("+propertyName+")");
			
			hqlAssembler.addParameter("param0", value);
		}
	}

	@Override
	public boolean isSatisfiedBy(Entity object) {
		if (!wasSet()) {
			return isSatisfiedIfNotSet();
		}
		try {
			List<Object> res = (List<Object>)PropertyUtils.getProperty(object, getPropertyName());
			return res.contains(value);
		} catch (IllegalAccessException e) {
			throw new RuntimeException();
		} catch (InvocationTargetException e) {
			throw new RuntimeException();
		} catch (NoSuchMethodException e) {
			throw new RuntimeException();
		}
	}

	@Override
	public boolean supportsAssembler(Assembler assembler) {
		return assembler instanceof HqlAssembler;
	}

	@Override
	public boolean wasSet() {
		return value != null ;
	}
	
	public Object getValue() {
		return value;
	}

	public void setValue(Object value) {
		this.value = value;
	}

	public String getPropertyName() {
		return propertyName;
	}

	public void setPropertyName(String propertyName) {
		this.propertyName = propertyName;
	}
}
