package it.amattioli.locate;

import java.util.Collection;
import java.util.Currency;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.apache.commons.lang.ObjectUtils;

import it.amattioli.dominate.Described;
import it.amattioli.dominate.EntityImpl;
import it.amattioli.dominate.Repository;
import it.amattioli.dominate.RepositoryRegistry;

public class Place extends EntityImpl implements Described {
	private PlaceType type;
	private String description;
	private String shortDescription;
	private Place parent;
	private Set<String> zipCodes = new TreeSet<String>();
	private Currency currency;
	private Locale locale;
	private Set<Organization> organizations = new HashSet<Organization>();
	private Map<String,String> code = new HashMap<String, String>();

	public PlaceType getType() {
		return type;
	}

	public void setType(PlaceType type) {
		this.type = type;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}
	
	public String getShortDescription() {
		return shortDescription;
	}

	public void setShortDescription(String shortDescription) {
		this.shortDescription = shortDescription;
	}
	
	public Place getTypedParent(String type) {
		return getParent(PlaceType.valueOf(type));
	}

	public Place getParent(PlaceType type) {
		if (ObjectUtils.equals(type, this.type)) {
			return this;
		}
		if (parent == null) {
			return null;
		}
		return parent.getParent(type);
	}

	public Place getParent() {
		return parent;
	}

	public void setParent(Place parent) {
		this.parent = parent;
	}

	public Set<String> getZipCodes() {
		return zipCodes;
	}
	
	public void addZipCode(String zipCode) {
		zipCodes.add(zipCode);
	}

	public Currency getCurrency() {
		if (currency == null && parent != null) {
			return parent.getCurrency();
		}
		return currency;
	}

	public void setCurrency(Currency currency) {
		this.currency = currency;
	}

	public Locale getLocale() {
		if (locale == null && parent != null) {
			return parent.getLocale();
		}
		return locale;
	}

	public void setLocale(Locale locale) {
		this.locale = locale;
	}

	public Collection<Organization> getOrganizations() {
		return organizations;
	}
	
	public void addOrganization(Organization organization) {
		if (!this.isMemberOf(organization)) {
			this.organizations.add(organization);
			organization.addMember(this);
		}
	}
	
	public boolean isMemberOf(Organization organization) {
		return this.organizations.contains(organization);
	}
	
	public String getCode(String codeName) {
		return code.get(codeName);
	}
	
	public void setCode(String codeName, String codeValue) {
		code.put(codeName, codeValue);
	}
	
	public static Repository<Long, Place> repository() {
		return RepositoryRegistry.instance().getRepository(Place.class);
	}
	
	public static Place fromCode(String codeName, String codeValue) {
		PlaceSpecification spec = new PlaceSpecification();
		spec.setCode(codeName, codeValue);
		List<Place> all = repository().list(spec);
		if (all.size() == 1) {
			return all.get(0);
		} else {
			return null;
		}
	}

}
