package it.amattioli.locate;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.lang.reflect.InvocationTargetException;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang.StringUtils;

import it.amattioli.applicate.context.hibernate.SameContext;
import it.amattioli.applicate.selection.SelectionEvent;
import it.amattioli.applicate.selection.SelectionListener;
import it.amattioli.dominate.hibernate.filters.ComparisonType;
import it.amattioli.dominate.util.PropertyChangeEmitter;
import it.amattioli.dominate.validation.DefaultValidator;
import it.amattioli.dominate.validation.ValidationResult;
import it.amattioli.dominate.validation.Validator;

public class PlaceSelector implements PropertyChangeEmitter {
	private static final String MATCH_PROPERTY = "match";
	private static final String SELECTED_OBJECT_PROPERTY = "selectedObject";
	private static final String DESCRIPTION_PROPERTY = "description";

	public static enum Match {NONE, EXACT, MULTIPLE};
	
	private Place selectedObject;
	private String description;
	private PlaceBrowser placeBrowser;
	private PropertyChangeSupport pChange = new PropertyChangeSupport(this);
	private SelectionListener placeBrowserSelectionListener;
	private Match match;
	private PropertyChangeListener placeListener;
	private Object boundBean;
	private String boundPropertyName;
	
	public PlaceSelector() {
		
	}
	
	public PlaceSelector(final PlaceBrowser placeBrowser) {
		setPlaceBrowser(placeBrowser);
	}
	
    public void addPropertyChangeListener(final PropertyChangeListener listener) {
        pChange.addPropertyChangeListener(listener);
    }

    public void removePropertyChangeListener(final PropertyChangeListener listener) {
        pChange.removePropertyChangeListener(listener);
    }

    public void firePropertyChange(final String propertyName, final Object oldValue, final Object newValue) {
        pChange.firePropertyChange(propertyName, oldValue, newValue);
    }
	
	public PlaceBrowser getPlaceBrowser() {
		return placeBrowser;
	}

	@SameContext
	public void setPlaceBrowser(final PlaceBrowser placeBrowser) {
		//this.placeBrowser = LongRunningContext.withCurrentSessionManager(placeBrowser);
		this.placeBrowser = placeBrowser;
		placeBrowserSelectionListener = new SelectionListener() {

			public void objectSelected(SelectionEvent event) {
				String old = description;
				Place selectedObject = placeBrowser.getSelectedObject();
				if (selectedObject != null) {
					description = selectedObject.getDescription();
					setMatch(Match.EXACT);
				} else {
					description = "";
				}
				firePropertyChange(DESCRIPTION_PROPERTY, old, description);
				setSelectedObject(selectedObject);
			}
			
		};
		placeBrowser.addSelectionListener(placeBrowserSelectionListener);
	}
	
	private void setSelectedObject(Place selectedObject) {
		Place old = this.selectedObject;
		this.selectedObject = selectedObject;
		firePropertyChange(SELECTED_OBJECT_PROPERTY, old, selectedObject);
	}

	public Place getSelectedObject() {
		return selectedObject;
	}
	
	public void setSelectedObjectId(Long id) {
		Place old = this.selectedObject;
		if (id == null) {
			this.selectedObject = null;
			this.description = "";
		} else {
			Place selected = Place.repository().get(id);
			this.selectedObject = selected;
			this.description = selected.getDescription();
		}
		setMatch(Match.EXACT);
		firePropertyChange(SELECTED_OBJECT_PROPERTY, old, selectedObject);
	}
	
	public Long getSelectedObjectId() {
		if (selectedObject == null) {
			return null;
		}
		return selectedObject.getId();
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(final String newDescription) {
		String description = StringUtils.upperCase(newDescription);
		String oldDescription = getDescription();
		if (!StringUtils.equals(oldDescription, description)) {
			Match oldMatch = getMatch();
			if (StringUtils.isBlank(description)) {
				this.description = "";
				setMatch(Match.EXACT);
				firePropertyChange(DESCRIPTION_PROPERTY, oldDescription, description);
				firePropertyChange(MATCH_PROPERTY, oldMatch, getMatch());
				placeBrowser.select((Place)null);
			} else {
				PlaceSpecification spec = placeBrowser.getSpecification();
				spec.setDescriptionType(ComparisonType.EXACT);
				spec.setDescription(description);
				placeBrowser.setSpecification(spec);
				if (placeBrowser.getList().size() == 0) {
					placeBrowser.getSpecification().setDescriptionType(ComparisonType.STARTS);
				}
				this.description = description;
				switch (placeBrowser.getList().size()) {
					case 0: 
						setMatch(Match.NONE);
					    break;
					
					case 1:
						setMatch(Match.EXACT);
						break;
						
					default:
						setMatch(Match.MULTIPLE);
					        
				}
				firePropertyChange(DESCRIPTION_PROPERTY, oldDescription, description);
				firePropertyChange(MATCH_PROPERTY, oldMatch, getMatch());
				if (placeBrowser.getList().size() == 1) {
					placeBrowser.select(0);
				}
			}
			
		}
	}
	
	private void setMatch(Match match) {
		this.match = match;
	}
	
	public Match getMatch() {
		return match;
	}
	
	public void setPlaceType(String placeType) {
		
	}
	
	public void bindTo(final Object bean, final String propertyName) {
		boundBean = bean;
		boundPropertyName = propertyName;
		placeListener = new PropertyChangeListener() {

			@Override
			public void propertyChange(PropertyChangeEvent evt) {
				if (SELECTED_OBJECT_PROPERTY.equals(evt.getPropertyName())) {
					try {
						PropertyUtils.setProperty(bean, propertyName, evt.getNewValue());
					} catch (IllegalAccessException e) {
						throw new RuntimeException(e);
					} catch (InvocationTargetException e) {
						throw new RuntimeException(e);
					} catch (NoSuchMethodException e) {
						throw new RuntimeException(e);
					}
				}
			}
			
		};
		addPropertyChangeListener(placeListener);
		Place initial;
		try {
			initial = (Place)PropertyUtils.getProperty(bean, propertyName);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		} catch (NoSuchMethodException e) {
			throw new RuntimeException(e);
		}
		if (initial != null) {
			setSelectedObjectId(initial.getId());
		} else {
			setSelectedObjectId(null);
		}
	}
	
	public ValidationResult validate() {
		if (boundBean != null) {
			Validator v = new DefaultValidator(boundBean);
			return v.validateProperty(boundPropertyName, getSelectedObject());
		} else {
			return null;
		}
	}
}
